SystemOrganization addCategory: #TusLibros!

TestCase subclass: #TestTusLibros
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TestTusLibros methodsFor: 'something' stamp: 'PabloRago 5/30/2013 17:37'!
aCartWithBooks

	| aCart aCatalog |
	
	aCatalog := Dictionary new.
	aCatalog at: self someBook put: 50.00.
	aCatalog at: self someOtherBook put: 69.99.
	
	aCart := Cart newWithCatalog: aCatalog.
	aCart add: self someBook.
	aCart add: self someBook.
	^aCart.		! !

!TestTusLibros methodsFor: 'something' stamp: 'PabloRago 5/30/2013 23:46'!
aCashier

	^ Cashier newWithSalesBook: (Set new).! !

!TestTusLibros methodsFor: 'something' stamp: 'PabloRago 5/30/2013 23:53'!
aCashierWithSalesBook: aSalesBook

	^ Cashier newWithSalesBook: aSalesBook! !

!TestTusLibros methodsFor: 'something' stamp: 'PabloRago 5/30/2013 17:40'!
aNewCart

	| aCatalog |
	
	aCatalog := Dictionary new.
	aCatalog at: self someBook put: 50.00.
	aCatalog at: self someOtherBook put: 69.99.
	
	^ Cart newWithCatalog: aCatalog.! !

!TestTusLibros methodsFor: 'something' stamp: 'PabloRago 5/30/2013 17:39'!
aNewCartWithEmptyCatalog

	^ Cart newWithCatalog: Dictionary new.! !

!TestTusLibros methodsFor: 'something' stamp: 'PabloRago 5/30/2013 17:38'!
aNonExpiredCreditCard

	^ CreditCard newWithOwner: 'Luis' withNumber: 123456 withExpirationDate: ((Date today) addDays: 60 ).! !

!TestTusLibros methodsFor: 'something' stamp: 'PabloRago 5/30/2013 17:18'!
anExpiredCreditCard

	^ CreditCard newWithOwner: 'Luis' withNumber: 123456 withExpirationDate: ((Date today) subtractDays: 60 ).! !

!TestTusLibros methodsFor: 'something' stamp: 'PabloRago 5/29/2013 19:48'!
someBook

	^ 123456.! !

!TestTusLibros methodsFor: 'something' stamp: 'PabloRago 5/29/2013 19:48'!
someOtherBook

	^ 987654.! !

!TestTusLibros methodsFor: '1 Tests Cart' stamp: 'PabloRago 5/27/2013 21:54'!
test01CreatedCartShouldBeEmpty
	
	
	self assert: (self aNewCart) isEmpty.! !

!TestTusLibros methodsFor: '1 Tests Cart' stamp: 'PabloRago 5/29/2013 19:39'!
test02BooksAddedToTheCartShouldBeInTheCart
	
	self assert: (((self aNewCart) add: self someBook) includes: self someBook).! !

!TestTusLibros methodsFor: '1 Tests Cart' stamp: 'PabloRago 5/29/2013 20:09'!
test03TheCartShouldOnlyContainTheAddedBooks
	
	| aCart |
	
	aCart := self aNewCart.
	aCart add: self someBook. 
	aCart add: self someOtherBook.

	self assert: (aCart includes: self someBook).
	self assert: (aCart includes: self someOtherBook).
	self assert: (aCart size = 2).! !

!TestTusLibros methodsFor: '1 Tests Cart' stamp: 'PabloRago 5/29/2013 19:39'!
test04TheCartCanContainTwoCopiesOfTheSameBook
	
	| aCart |
	
	aCart := self aNewCart.
	aCart add: self someBook. 
	aCart add: self someBook.

	self assert: (aCart includes: self someBook).
	self assert: (aCart size = 2).! !

!TestTusLibros methodsFor: '1 Tests Cart' stamp: 'PabloRago 5/31/2013 00:09'!
test05TheCartCanContainManyCopiesOfManyBooks
	
	| aCart |
	
	aCart := self aNewCart.
	
	aCart add: self someBook. 
	aCart add: self someBook.
	aCart add: self someOtherBook.
	aCart add: self someOtherBook.
	aCart add: self someOtherBook.
	
	self assert: ((aCart occurrencesOf: self someBook) = 2).
	self assert: ((aCart occurrencesOf: self someOtherBook) = 3).! !

!TestTusLibros methodsFor: '1 Tests Cart' stamp: 'PabloRago 5/31/2013 00:10'!
test06ItemsThatAreNotInTheCatalogCannotBeAddedToTheCart
	
	| aCart |
	
	aCart := self aNewCartWithEmptyCatalog .
	
	self should: [ aCart add:  self someBook ] 
	raise: Error 
	withExceptionDo: [ :anError | self assert: (anError description = ('Error: ', Cart invalidItemErrorMessage)) ].
		
	self assert: aCart isEmpty. ! !

!TestTusLibros methodsFor: '2 Test checkout' stamp: 'PabloRago 5/31/2013 00:05'!
test07AnEmptyCartCannotBeCheckedOut

	|aSalesBook aCashier|
	
	aSalesBook := Set new.
	aCashier := self aCashierWithSalesBook: aSalesBook.

	self should: [ 
		aCashier checkout: (self aNewCart) withCreditCard: (self aNonExpiredCreditCard) 
		]
	raise: Error 
	withExceptionDo: [ :anError | self assert: (anError description = ('Error: ', Cashier emptyCartCheckoutErrorMessage)) ].
		
	self assert: (self aNewCart isEmpty).
	self assert: ( aSalesBook isEmpty  ).! !

!TestTusLibros methodsFor: '2 Test checkout' stamp: 'PabloRago 5/31/2013 00:06'!
test08ACashierShouldNotAcceptAnExpiredCreditCard

	|aSalesBook aCashier|
	
	aSalesBook := Set new.
	aCashier := self aCashierWithSalesBook: aSalesBook.
	
	self should: [ 
		aCashier checkout: (self aCartWithBooks) withCreditCard: (self anExpiredCreditCard) 
		]
	raise: Error 
	withExceptionDo: [ :anError | self assert: (anError description = ('Error: ', Cashier expiredCreditCardErrorMessage)) ].

	self assert: ( aSalesBook isEmpty  ).! !

!TestTusLibros methodsFor: '2 Test checkout' stamp: 'PabloRago 5/31/2013 00:08'!
test09SaleAmountShouldBeEqualToTheTotalPriceOfThePurchasedBooks

	| aCart aCatalog aSale aSalesBook aCashier|

	aSalesBook := Set new.
	aCashier := self aCashierWithSalesBook: aSalesBook.
	
	aCatalog := Dictionary new.
	aCatalog at: (self someBook)  put: 50.00.
	aCatalog at: (self someOtherBook)  put: 69.99.
	
	aCart := Cart newWithCatalog: aCatalog.
	aCart add: (self someBook). 
	aCart add: (self someBook).
	aCart add: (self someBook).
	
	aSale := aCashier checkout: aCart withCreditCard: (self aNonExpiredCreditCard) .
	
	self assert: (aSale totalAmount = 150.00).
	self assert: (aSalesBook includes: aSale).
		
	

	! !

!TestTusLibros methodsFor: 'n TestBatchProcessor' stamp: 'PabloRago 6/1/2013 20:59'!
testFile

	| fileStream |
	fileStream := FileStream fileNamed: 'batch.csv'.
	
	fileStream close.! !

!TestTusLibros methodsFor: 'n TestBatchProcessor' stamp: 'PabloRago 6/1/2013 18:52'!
testTest

	| input line stream char word text b|
	
	input := (String crlf join: #( '1,2,3' '4,5,6' )).

	"batch := BatchProcessor newWithInputStream: (input readStream ) withOutputStream: 8."
	
	"line := input readStream upTo: (String crlf at: 1).
	
	 b := 5."
	
	stream := input readStream.
	word := ''.
	line := LinkedList  new.
	text := LinkedList new.
	
	[stream atEnd] whileFalse: [ 
			char := stream next.
			(char = $,) ifTrue: [ 
					"Se termino la palabra entonces la agrego a la linea"
					line add: word.
					word := ''.
				 ] ifFalse: [ 
			(char = (String crlf at: 1)) ifTrue: [ 
					"Llegu a un enter, por lo qe es nueva linea, yl agrego al texto"
					stream next. "remove lf".
					line add: word.
					word := ''.
					text add: line.
					line := LinkedList new.
										
				 ] ifFalse: [
					"Es un caracter, lo agrego a la plabra actual, es decir todavia no termino"
					word := word , char asString. 
				]
			]			  
		 ].
	line add: word.
	text add: line.
	
	b :=5.! !

Object subclass: #BatchProcessor
	instanceVariableNames: 'inputStream outputStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!BatchProcessor class methodsFor: 'as yet unclassified' stamp: 'PabloRago 6/1/2013 15:49'!
newWithInputStream: anInputStream withOutputStream: anOutputStream

	| aBatchProcessor |
	
	aBatchProcessor := BatchProcessor new.
	^aBatchProcessor initializeWithInputStream: anInputStream withOutputStream: anOutputStream.! !

!BatchProcessor methodsFor: 'as yet unclassified' stamp: 'PabloRago 6/1/2013 17:26'!
initializeWithInputStream: anInputStream withOutputStream: anOutputStream.

	inputStream := anInputStream.
	outputStream := anOutputStream.! !

!BatchProcessor methodsFor: 'as yet unclassified' stamp: 'PabloRago 6/1/2013 20:44'!
processInput

	|parsedInput aCart aCashier aCreditCard aSale|

	parsedInput := CSVHandler readCSV: inputStream .
	aCart := Cart newWithCatalog: EditorialLosPelados defaultCatalog. 
	parsedInput do:[:aLine |
				(aLine at:1 = '1') ifTrue:[
					aCart add: (aLine at:2) asInteger withOccurrences: (aLine at:3) asInteger.
				 ]ifFalse:[
					aCashier := Cashier new.
					aCreditCard := CreditCard newWithOwner: (aLine at:4)  withNumber: ((aLine at:2) asInteger) withExpirationDate: (aLine at:3) asInteger.

					[aSale := aCashier checkout: aCart withCreditCard: aCreditCard. "Tengo el transaction id"
						self persistSaleSuccessWithTransactionID: (aSale transactionId) withTotal: (aSale totalAmount) .
					] on: Error do:  [ :anError |
						self persistSaleFailWithError: (anError message) 
					].

					aCart := Cart newWithCatalog: EditorialLosPelados defaultCatalog. 
				]		
	]  ! !

Object subclass: #CSVHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CSVHandler class methodsFor: 'as yet unclassified' stamp: 'PabloRago 6/1/2013 20:40'!
readCSV: inputStream

	| stream word line text char |
	stream := inputStream readStream.
	word := ''.
	line := LinkedList  new.
	text := LinkedList new.
	
	[stream atEnd] whileFalse: [ 
			char := stream next.
			(char = $,) ifTrue: [ 
					"Se termino la palabra entonces la agrego a la linea"
					line add: word.
					word := ''.
				 ] ifFalse: [ 
			(char = (String crlf at: 1)) ifTrue: [ 
					"Llegu a un enter, por lo qe es nueva linea, y la agrego al texto"
					stream next. "remove lf".
					line add: word.
					word := ''.
					text add: line.
					line := LinkedList new.
										
				 ] ifFalse: [
					"Es un caracter, lo agrego a la plabra actual, es decir todavia no termino"
					word := word , char asString. 
				]
			]			  
		 ].
	line add: word.
	text add: line.
	
	^text.! !

!CSVHandler class methodsFor: 'as yet unclassified' stamp: 'PabloRago 6/1/2013 21:47'!
writeCSV: anArray

	|writeStream|

	writeStream := WriteStream  with: String new. 
	anArray do: [ :aLine | 
		aLine do: [ :aWord |
			writeStream nextPut: aWord.
			writeStream nextPut: $,.			
		 ].
		writeStream position: writeStream size.
		writeStream nextPut: String crlf.
	 ].
	

	^ writeStream.! !

Object subclass: #Cart
	instanceVariableNames: 'bag catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cart class methodsFor: 'as yet unclassified' stamp: 'PabloRago 5/29/2013 20:28'!
invalidItemErrorMessage

	^ 'The item cannot be added.'! !

!Cart class methodsFor: 'as yet unclassified' stamp: 'PabloRago 5/29/2013 20:22'!
newWithCatalog: aCatalog

	| aCart |
	
	aCart := Cart new.
	^ aCart initializeWithCatalog: aCatalog.! !

!Cart methodsFor: 'adding' stamp: 'PabloRago 5/30/2013 17:44'!
add: anElement 

	(catalog includesKey: anElement) 
	ifTrue: 
		[bag add: anElement ]
	ifFalse: 
		[ ^ Error signal: (self class invalidItemErrorMessage)  ].
	! !

!Cart methodsFor: 'accessing' stamp: 'PabloRago 5/29/2013 19:55'!
includes: anElement 

	^bag includes:  anElement .! !

!Cart methodsFor: 'initialization' stamp: 'PabloRago 5/29/2013 21:23'!
initializeWithCatalog: aCatalog 

	catalog := aCatalog.
	bag := Bag new.! !

!Cart methodsFor: 'accessing' stamp: 'PabloRago 5/29/2013 19:55'!
isEmpty

	^ bag isEmpty.! !

!Cart methodsFor: 'accessing' stamp: 'PabloRago 5/29/2013 19:55'!
occurrencesOf: anElement 

	^bag occurrencesOf:  anElement .! !

!Cart methodsFor: 'accessing' stamp: 'PabloRago 5/29/2013 19:56'!
size 

	^ bag size.! !

!Cart methodsFor: 'accessing' stamp: 'PabloRago 5/31/2013 00:01'!
totalAmount

	| totalAmount |
	
	totalAmount := 0.
	bag do: [ :anItem | totalAmount := totalAmount + (catalog at: anItem) ].
	^totalAmount.
	
	"^(bag reduce:  [ :anItem :anotherItem | (catalog at: anItem) + (catalog at: anotherItem) ])."! !

Object subclass: #Cashier
	instanceVariableNames: 'salesBook'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cashier class methodsFor: 'as yet unclassified' stamp: 'PabloRago 5/29/2013 21:08'!
emptyCartCheckoutErrorMessage

	^ 'Cannot checkout an empty cart.'! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'PabloRago 5/30/2013 17:25'!
expiredCreditCardErrorMessage

	^ 'Cannot checkout with an expired credit card.'! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'PabloRago 5/30/2013 17:19'!
expredCreditCardErrorMessage

	^ 'Cannot checkout with an expired credit card.'! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'PabloRago 5/30/2013 23:45'!
newWithSalesBook: aSalesBook 

	|aCashier|
	
	aCashier := Cashier new.
	^ aCashier initializeWithSalesBook: aSalesBook.! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'PabloRago 6/1/2013 20:32'!
checkout: aCart withCreditCard: aCreditCard
	
	| aSale saleTotalAmount aTransactionId|
	
	self signalIfCartIsEmptyFor: aCart.
	self signalIfCreditCardIsExpiredFor: aCreditCard.
	
	saleTotalAmount := aCart totalAmount.
	
	[aTransactionId := MerchantProcessorInterface debitWithCreditCard: aCreditCard  andTotal: aCart.
	aSale := Sale newWithCart: aCart withCreditCard: aCreditCard withTransactionID: aTransactionId for: saleTotalAmount.  
	salesBook add: aSale.
	^aSale.] 
		on: Error
		do: [ :anError | 
				"El unico error que handleamo' es Timeout, viteh"
				(anError message = 'Timeout') ifTrue: [self persistCSV.] .
				 self signal: anError.
			].
	
	
	
	
	 ! !

!Cashier methodsFor: 'initialization' stamp: 'PabloRago 6/1/2013 19:13'!
initialize

	salesBook := Set new.! !

!Cashier methodsFor: 'initialization' stamp: 'PabloRago 5/30/2013 23:44'!
initializeWithSalesBook: aSalesBook

	salesBook := aSalesBook.! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'PabloRago 5/30/2013 23:50'!
signalIfCartIsEmptyFor: aCart
	
	(aCart isEmpty)
	ifTrue: 
		[ ^ Error signal: (self class emptyCartCheckoutErrorMessage)  ].
	
	
	 ! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'PabloRago 5/30/2013 23:49'!
signalIfCreditCardIsExpiredFor: aCreditCard
	
	(aCreditCard expirationDate < Date today)
	ifTrue: 
		[ ^ Error signal: (self class expiredCreditCardErrorMessage)  ].
	
	 ! !

Object subclass: #CreditCard
	instanceVariableNames: 'owner number expirationDate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CreditCard class methodsFor: 'as yet unclassified' stamp: 'PabloRago 5/29/2013 21:21'!
newWithOwner: anOwner withNumber: aNumber withExpirationDate: anExpirationDate

	|aCreditCard|
	
	aCreditCard := CreditCard new.
	^ aCreditCard initializeWithOwner: anOwner withNumber: aNumber withExpirationDate: anExpirationDate ! !

!CreditCard methodsFor: 'accessors' stamp: 'PabloRago 5/30/2013 17:22'!
expirationDate

	^ expirationDate.! !

!CreditCard methodsFor: 'initialization' stamp: 'PabloRago 5/29/2013 21:24'!
initializeWithOwner: anOwner withNumber: aNumber withExpirationDate: anExpirationDate 

	owner := anOwner.
	number := aNumber .
	expirationDate := anExpirationDate .

! !

!CreditCard methodsFor: 'accessors' stamp: 'PabloRago 5/30/2013 17:22'!
number

	^ number.! !

!CreditCard methodsFor: 'accessors' stamp: 'PabloRago 5/30/2013 17:22'!
owner

	^ owner.! !

Object subclass: #EditorialLosPelados
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!EditorialLosPelados class methodsFor: 'as yet unclassified' stamp: 'PabloRago 6/1/2013 19:06'!
defaultCatalog

	|aCatalog|
	
	aCatalog := Dictionary new.
	aCatalog at: 123  put: 50.00.
	aCatalog at: 124  put: 69.99.
	
 	^aCatalog.! !

Object subclass: #MerchantProcessorInterface
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!MerchantProcessorInterface class methodsFor: 'as yet unclassified' stamp: 'PabloRago 6/1/2013 19:40'!
debitWithCreditCard: aCreditCard andTotal: total 

	^1
	"( response status = 400) 
		signal Mandaste cualquiera
	( response status = 408 ) 
		signal timeout
	 ( status = 200)
		if content = OK, ^transactionId
		else signal Error:Hubo error loco 
"! !

Object subclass: #Sale
	instanceVariableNames: 'cart creditCard totalAmount date transactionID'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Sale class methodsFor: 'as yet unclassified' stamp: 'PabloRago 6/1/2013 19:56'!
newWithCart: aCart withCreditCard: aCreditCard withTransactionID: aTransactionID for: saleTotalAmount

	| aSale |
	
	aSale := Sale new.
	^ aSale initializeWithCart: aCart withCreditCard: aCreditCard withTransactionID: aTransactionID for: saleTotalAmount.	
! !

!Sale methodsFor: 'as yet unclassified' stamp: 'PabloRago 6/1/2013 20:32'!
initializeWithCart: aCart withCreditCard: aCreditCard withTransactionID: aTransactionID for: aTotalAmount

	cart := aCart.
	creditCard := aCreditCard.
	totalAmount := aTotalAmount .
	date := Date dateAndTimeNow.
	transactionID := aTransactionID.! !

!Sale methodsFor: 'as yet unclassified' stamp: 'PabloRago 5/30/2013 23:35'!
totalAmount

	^totalAmount.! !

!Sale methodsFor: 'as yet unclassified' stamp: 'PabloRago 6/1/2013 20:32'!
transactionID

	^transactionID.! !
